/*
 * @ReadFluidLevelDataXMLFile.java Version 1.0 03/21/2016
 *
 * Copyright (c) 2016 John R. Victorine
 * All Rights Reserved.
 */

package fluid.io;

import java.util.*;
import java.io.*;
import java.net.*;
import java.awt.*;
import java.util.zip.*;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;

import cmn.cmnString;
import fluid.fluidListStruct;
import fluid.fluidStruct;

/** Class ReadFluidLevelDataXMLFile()
 * <p> This Class will parse the Measured Brine XML Data Stream
 *     directly to the Measured Brine List Data Structure.
 *
 *  @version 1.1 03/21/2016
 *  @author  John R. Victorine
 */

public class ReadFluidLevelDataXMLFile extends DefaultHandler
{
  public static final int FILE       = 0;
  public static final int URL        = 1;
  public static final int SERVER     = 2;
  public static final int SERVER_ZIP = 3;

  private int              iType  = FILE;
  private int              iRows  = 0;
  private int              iCount = -1;

  private int              iComments = 0;

  public static final String ROOTNAME = "fluid_data"; // ROOT NAME
  public static final String RECORDS  = "records";    // Attribute

  // Brine Data

  public static final String DATA     = "data";    // ELEMENT
  public static final String ROW      = "row";     // ELEMENT

  public static final String KEY      = "KEY";     // "Fluid Primary Key"
  public static final String STATE    = "STATE";   // "State"
  public static final String COUNTY   = "COUNTY";  // "County"
  public static final String FIELD    = "FIELD";   // "Field"
  public static final String LOC      = "LOC";     // "Location"
  public static final String NAMEA    = "NAME";    //  Name of Well
  public static final String API      = "API";     // "Well API-Number"
  public static final String WELL_KID = "WELL_KID"; // "Well Header KID"

  public static final String LAT      = "LAT";     // "Latitude"
  public static final String LONG     = "LONG";    // "Longitude"
  public static final String UTM_X    = "UTM_X";   // "UTM-X"
  public static final String UTM_Y    = "UTM_Y";   // "UTM-Y"
  public static final String UTM_Z    = "UTM_Z";   // "UTM Zone"

  public static final String KB       = "KB";      // "Kelly Bushing"
  public static final String GL       = "GL";      // "Ground Level"
  public static final String DF       = "DF";      // "Derrick Floor"
  public static final String TGT      = "TGT";     // "TGT"

  public static final String FORM     = "FORM";    // "Formation"
  public static final String AGE      = "AGE";     // "Formation Age"

  public static final String SRC      = "SRC";     // "Sample Source"
  public static final String DATE     = "DATE";    // "Sample Date"

  public static final String LAB      = "LAB";     // "Laboratory Name"
  public static final String ADDRESS  = "ADDRESS"; // "Laboratory Address"
  public static final String CITY     = "CITY";    // "Laboratory City"
  public static final String STATE_L  = "STATE_L"; // "Laboratory State"
  public static final String ZIPCODE  = "ZIPCODE"; // "Laboratory Zipcode"

  public static final String TOP      = "TOP";     // "Depth Top" unit="ft"
  public static final String BASE     = "BASE";    // "Depth Base" unit="ft"

  public static final String ID       = "ID";      // Group ID - LAB or FIELD
  public static final String NO       = "NO";      // Group Number

  public static final String TOTAL    = "TOTAL";   // Total Fluid
  public static final String CUT      = "CUT";     // Oil Cut (%)
  public static final String LEVEL    = "LEVEL";   // Fluid Level (ft)
  public static final String JOINTS   = "JOINTS";  // Joints in Well
  public static final String BOPD     = "BOPD";    // Barrels of Oil ber day
  public static final String BHP      = "BHP";     // Estimated Bottom Hole Pressure (psig)
  public static final String CO2      = "CO2";     // CO2 (%)
  public static final String MCF      = "MCF";     // GAS (mdf/d)

  private static final int   _NONE    = 0;
  private static final int   _DATA    = 1;
  private static final int   _ROW     = 2;

  private int iData = _NONE;

  private fluidListStruct stList = new fluidListStruct();

  private int    iError = 0;
  private String sError = "";

  /** Method ReadFluidLevelDataXMLFile()
   *  This is the Default Constructor for this class.
   */

  public ReadFluidLevelDataXMLFile() { iType = FILE; }

  /** Method ReadFluidLevelDataXMLFile()
   *  This is the Constructor for this class.
   *  @param iType = The source identifier,
   *                 0 = File is an absolute Directory Path + Filename
   *                 1 = Source is a URL Location.
   *                 2 = Server Location
   *                 3 = WebStart Location
   */

  public ReadFluidLevelDataXMLFile(int iType) { this.iType = iType; }

  /** Method Process()
   *  <p> This method will Process the XML File into a Array List.
   *      It is a double pass method in that it Reads the XML File
   *      once to get the number of lines in the XML File and a
   *      second time to actually read the XML File.
   *  @param  filename = The File to be opened.
   *  @return stList   = The XML List Data Structure (Array List).
   */

  public fluidListStruct Process(String filename)
  {
 	double dTotal = 0.0;
 	double data   = 0.0;
 	int    iTotal = 0;

    Read(filename);

    return (stList);
  }

  /** Method Read()
   *  <p> This method will Read & Parse the XML File.
   *  @param filename = the filename of the XML File.
   */

  public void Read(String filename)
  {
    // Use the validating parser

    SAXParserFactory factory = SAXParserFactory.newInstance();
    factory.setValidating(true);

    try
    {
      SAXParser saxParser = factory.newSAXParser();
      if (iType == FILE)
      {
        saxParser.parse( new File(filename), this);
      }
      else if (iType == URL)
      {
        try
        {
          InputSource source = new InputSource(filename);
          saxParser.parse(source, this);
        }
        catch (Exception e)
        {
          System.err.println("ReadFluidLevelDataXMLFile.Read() " + e);
          e.printStackTrace();
        }
      }
      else if (iType == SERVER)
      {
        try
        {
          // Connect to the server

          URL u = new URL(filename);
          URLConnection uc = u.openConnection();
          HttpURLConnection connection = (HttpURLConnection) uc;
          connection.setDoOutput(true);
          connection.setDoInput(true);
          connection.setRequestMethod("POST");

          // Read the response XML Document

          InputStream in = connection.getInputStream();
          InputSource source = new InputSource(in);
          saxParser.parse(source, this);
          in.close();
          connection.disconnect();
        }
        catch (Exception e)
        {
          System.err.println("ReadFluidLevelDataXMLFile.Read() " + e);
          e.printStackTrace();
        }
      }
      else
      {
        try
        {
          // Connect to the server

          URL u = new URL(filename);
          URLConnection uc = u.openConnection();
          HttpURLConnection connection = (HttpURLConnection) uc;
          connection.setDoOutput(true);
          connection.setDoInput(true);
          connection.setRequestMethod("POST");

          // Read the response XML Document

          GZIPInputStream in = new GZIPInputStream(connection.getInputStream());
          InputSource source = new InputSource(in);
          saxParser.parse(source, this);
          in.close();
          connection.disconnect();
        }
        catch (Exception e)
        {
          System.err.println(e);
        }
      }
    }
    catch (SAXParseException spe)
    {
      // Error generated by the parser

      System.out.println("\n** Parsing error"            +
                         ", line " + spe.getLineNumber() +
                         ", uri "  + spe.getSystemId());
      System.out.println("   " + spe.getMessage() );

      // Use the contained exception, if any

      Exception  x = spe;
      if (spe.getException() != null)
        x = spe.getException();
      x.printStackTrace();
    }
    catch (SAXException sxe)
    {
      // Error generated by this application (or a parser-initialization error)

      Exception  x = sxe;
      if (sxe.getException() != null)
        x = sxe.getException();
      x.printStackTrace();
    }
    catch (ParserConfigurationException pce)
    {
      // Parser with specified options can't be built

      pce.printStackTrace();
    }
    catch (IOException ioe)
    {
      // I/O error

      ioe.printStackTrace();
    }
  }

  //===========================================================
  // SAX DocumentHandler methods
  //===========================================================

  /** Method setDocumentLocator()
   * <p> This is the Set Document Locator Method
   * @param l Document Locator
   */

  public void setDocumentLocator(Locator l)
  {
    // Ignore errors
  }

  /** Method startDocument()
   * <p> This is the Start Document Method
   * @throws SAXException
   */

  public void startDocument() throws SAXException
  {
    // Ignore it
  }

  /** Method endDocument()
   * <p> This is the End Document Method
   * @throws SAXException
   */

  public void endDocument() throws SAXException
  {
    // Ignore it
  }

  /** Method startElement()
   * <p> This method will parse the ELEMENT from an XML File
   * @param namespaceURI = Name Space URI
   * @param lName        = Local Name
   * @param qName        = Qualified Name
   * @param attrs        = Attributes
   * @throws SAXException
   */

  public void startElement(String namespaceURI,
                           String lName, // local name
                           String qName, // qualified name
                           Attributes attrs) throws SAXException
  {
    String eName = lName; // element name

    if ("".equals(eName))
      eName = qName; // namespaceAware = false

    // If the ELEMENT Well is detected then initialize the structure.

    if (eName.equals(DATA))
    {
      iRows  = 0;
      iCount = -1;
      iData  = _DATA;
    }

    if (eName.equals(ROW))
    {
      iCount++;
      if (iCount < iRows)
      {
        stList.stItem[iCount] = new fluidStruct();
      }
      iData  = _ROW;
    }

    // Add Attributes to the Well Header Data List Structure

    if (attrs != null)
    {
      for (int i=0; i < attrs.getLength(); i++)
      {
        String aName = attrs.getLocalName(i); // Attr name
        if ("".equals(aName))
          aName = attrs.getQName(i);

        String sTemp = new String(attrs.getValue(i));
        String sOut  = removeExcess(sTemp);

        // Get the total number of records available

        if (aName.equals(RECORDS))
        {
          switch (iData)
          {
            case _NONE:
              break;

            case _DATA:
              iRows = cmn.cmnString.stringToInt(sOut.trim());
              stList.stItem = new fluidStruct[iRows];
              stList.iCount = iRows;
              break;
          }
        }

        // Parse each attribute and add it to the structure

        if (iCount > -1)
        {
          switch (iData)
          {
            case _NONE:
              break;

            case _ROW:
              if (iCount < iRows)
              {
                stList.stItem[iCount] = ParseData( aName,
                                                   sOut.trim(),
                                                   stList.stItem[iCount] );
              }
              break;
          }
        }

        // End Add Attributes to List
      }
    }
  }

  /** Method endElement()
   * <p> This method will parse the ELEMENT from an XML File
   * @param namespaceURI = Name Space URI
   * @param sName        = Simple Name
   * @param qName        = Qualified Name
   * @throws SAXException
   */

  public void endElement(String namespaceURI,
                         String sName, // simple name
                         String qName  // qualified name
                        ) throws SAXException
  {
    // Ignore it
  }

  /** Method characters()
   * <p> This method will parse the CHARACTERS from an XML File
   * @param buf    = Buffer Character Array holding the characters
   * @param offset = String Offset
   * @param len    = Length of String
   * @throws SAXException
   */

  public void characters(char buf[], int offset, int len) throws SAXException
  {
    String s = new String(buf, offset, len);
  }

  /** Method ignorableWhitespace()
   * <p> This method will parse the Ignorable White Space from an XML File
   * @param buf    = Buffer Character Array holding the characters
   * @param offset = String Offset
   * @param len    = Length of String
   * @throws SAXException
   */

  public void ignorableWhitespace(char buf[], int offset, int len)
    throws SAXException
  {
    // Ignore it
  }

  /** Method processingInstruction()
   * <p> This method will processing Instructions for a XML File
   * @param target = Target
   * @param data   = Data
   * @throws SAXException
   */

  public void processingInstruction(String target, String data)
    throws SAXException
  {
    // Ignore it
  }

  //===========================================================
  // SAX ErrorHandler methods
  //===========================================================

  /** Method error()
   * <p> This method will treat validation errors as fatal
   * @param e = SAX Parse Exception
   * @throws SAXParseException
   */

  public void error(SAXParseException e) throws SAXParseException { throw e; }

  /** Method warning()
   * <p> This method will treat warnings
   * @param err = SAX Parse Exception
   * @throws SAXParseException
   */

  public void warning(SAXParseException err) throws SAXParseException
  {
    System.out.println("** Warning" +
                       ", line "    + err.getLineNumber() +
                       ", uri "     + err.getSystemId());
    System.out.println("   " + err.getMessage());
  }

  //===========================================================
  // Utility Methods ...
  //===========================================================

  /** Method removeExcess()
   * <p> Remove excess white space within a string
   * @param  sin  = String to be parsed of excess spaces
   * @return sout = String with the excess spaces removed
   */

  private String removeExcess(String sin)
  {
    String sout   = new String("");
    char   chold  = ' ';
    char   ch[];

    ch = sin.toCharArray();

    for (int i=0; i<ch.length; i++)
    {
      if (i==0)
        chold = ch[i];

      if (chold != ' ')
      {
        sout = new String(sout + ch[i]);
      }
      else if (chold == ' ')
      {
        if (ch[i] != ' ')
        {
          sout = new String(sout + ch[i]);
        }
      }

      chold = ch[i];
    }

    return (sout);
  }

  /** ParseData()
   * <p> This method will parse the Data Stream
   * @param  sIdentifier = The Attribute Identifier
   * @param  sData       = Measured Brine Data String
   * @param  st          = Brine Data Structure
   * @return st          = Brine Data Structure
   */

  private fluidStruct ParseData( String      sIdentifier,
                                 String      sData,
                                 fluidStruct st )
  {
    String          sTemp     = "";
    int             iCount    = 0;
    double          data      = 0.0;
    int             iContinue = 0;

    if (sIdentifier.equals(KEY))
    {
	  stList._KEY++;
	  st.sKID = new String(sData);
	  st.sKEY = new String(sData);
	}

    if (sIdentifier.equals(ID))      // Group ID - FLUID
    {
      st.group_id = new String(sData);
    }
    if (sIdentifier.equals(NO))      // Group Number
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        st.group_no = cmn.cmnString.stringToInt(sData);
        st.sKID     = new String(st.sKID+"_"+sData);
        st.sKEY     = new String(st.sKEY+"_"+sData);
      }
	}

    if (sIdentifier.equals(STATE))  // "State"
    {
      st.state = new String(sData);
    }
    if (sIdentifier.equals(COUNTY)) // "County"
    {
      st.sCounty = new String(sData);
    }
    if (sIdentifier.equals(FIELD))  // "Field"
    {
      st.sField = new String(sData);
    }
    if (sIdentifier.equals(LOC))  // "Location"
    {
      st.sLocation = new String(sData);
    }
    if (sIdentifier.equals(API))    // "Well API-Number"
    {
      st.sAPI = new String(sData);
    }
    if (sIdentifier.equals(NAMEA))   // "Well Name"
    {
      st.sName = new String(sData);
    }

    if (sIdentifier.equals(WELL_KID)) // "Well Header KID"
    {
      st.sWellKID = new String(sData);
	}

    if (sIdentifier.equals(LAT))    // "Latitude"
    {
      if (iContinue == 1)
      {
        st.dLatitude = data;
      }
    }
    if (sIdentifier.equals(LONG))   // "Longitude"
    {
      if (iContinue == 1)
      {
        st.dLongitude = data;
      }
    }

    if (sIdentifier.equals(UTM_Z))
    {
      st.dZone = cmn.cmnString.stringToDouble(sData);
    }
    if (sIdentifier.equals(UTM_X))
    {
      st.dUTMx = cmn.cmnString.stringToDouble(sData);
    }
    if (sIdentifier.equals(UTM_Y))
    {
      st.dUTMy = cmn.cmnString.stringToDouble(sData);
    }

    if (sIdentifier.equals(KB))   // "Kelly Bushing"
    {
      if (iContinue == 1)
      {
        st.dKB = data;
      }
    }
    if (sIdentifier.equals(GL))   // "Ground Level"
    {
      if (iContinue == 1)
      {
        st.dGL = data;
      }
    }
    if (sIdentifier.equals(DF))   // "Derrick Floor"
    {
      if (iContinue == 1)
      {
        st.dDF = data;
      }
    }
    if (sIdentifier.equals(TGT))   // "TGT"
    {
      if (iContinue == 1)
      {
        st.dTGT = data;
      }
    }

    if (sIdentifier.equals(SRC))    // "Sample Source"
    {
      st.source = new String(sData);
    }
    if (sIdentifier.equals(DATE))   // "Sample Date"
    {
      st.sRecovery = new String(sData);
    }

    if (sIdentifier.equals(TOTAL))   // Total Fluid
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stList._FLUID++;
		st.dTotal = cmn.cmnString.stringToDouble(sData);
	  }
	}

    if (sIdentifier.equals(CUT))    // Oil Cut (%)
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stList._CUT++;
		st.dOilCut = cmn.cmnString.stringToDouble(sData);
	  }
	}

    if (sIdentifier.equals(LEVEL))   // Fluid Level (ft)
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stList._LEVEL++;
		st.dLevel = cmn.cmnString.stringToDouble(sData);
	  }
	}

    if (sIdentifier.equals(JOINTS)) // Joints in Well
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stList._JNTS++;
		st.dJoints = cmn.cmnString.stringToDouble(sData);
	  }
	}

    if (sIdentifier.equals(BOPD))    // Barrels of Oil ber day
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stList._BOPD++;
		st.dBOPD = cmn.cmnString.stringToDouble(sData);
	  }
	}

    if (sIdentifier.equals(BHP))    // Estimated Bottom Hole Pressure (psig)
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stList._BHP++;
		st.dBHP = cmn.cmnString.stringToDouble(sData);
	  }
	}

    if (sIdentifier.equals(CO2))    // CO2 (%)
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stList._CO2++;
		st.dCO2 = cmn.cmnString.stringToDouble(sData);
	  }
	}

    if (sIdentifier.equals(MCF))     // GAS (mdf/d)
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stList._MCFD++;
		st.dMCFD = cmn.cmnString.stringToDouble(sData);
	  }
	}

    return (st);
  }

  /* --------------------------------------------------------------- *
   * ------------------- ERROR HANDLING METHODS -------------------- *
   * --------------------------------------------------------------- */

  /** METHOD GetErrorID()
   *  <p> This method will return the error number.
   * @return iError = The Error Identifier
   */

  public int GetErrorID() { return (iError); }

  /** METHOD GetError()
   *  <p> This method will return the error string.
   * @return sError = The Error Text
   */

  public String GetError() { return (sError); }
}

/*
 *  @version 1.1 03/21/2016
 *  @author  John Victorine
 */
