/*
 * @gridControlFrame.java Version 1.1 02/25/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package grid.gui;

import java.util.Observer;
import java.util.Observable;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import grid.gridStruct;                     // Grid Structure
import grid.gridDataStruct;                 // Grid Data Structure
import grid.gui.gridControlDataPanel;       // Grid Control Data Panel
import grid.plot.gridPlotMappingListStruct; // Grid Plot Mapping List Structure
import grid.plot.gridAnimationFrame;        // Grid Animation Frame

//import brine.brineListStruct;               // Brine List Structure

/** Class gridControlFrame
 *  <p> This Class will create the Stiff Frame.
 *
 *  @version 1.1 02/25/2016
 *  @author  John R. Victorine
 */

public class gridControlFrame extends JFrame implements ActionListener, Observer
{
  private static final int   _RAW    = 0;
  private static final int   _NORMAL = 1;

  // Input Variables

  private String               sID      = null;
  private gridStruct           stStruct = null;
  private gridDataStruct       stData   = null;

  // Global Variables

  private Observable           notifier = null; // Observables Object

  private int                  iWidth   = 650;

  private gridControlDataPanel pData    = null;
  private gridAnimationFrame   plot     = null;

  // Global Widgets

  private JButton btnClose     = new JButton();
  private JButton btnPlot      = new JButton();

  /** Constructor gridControlFrame()
   *  <p> This is the Constructor for this class.
   *  @param st  = Brine Plot Data Structure
   */

  public gridControlFrame( String sID, gridStruct stStruct, gridDataStruct stData )
  {
    try
    {
	  this.sID      = sID;
	  this.stStruct = stStruct;
      this.stData   = stData;

      jbInit();
      setButtons();

      addWindowListener( new gridControlFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	JPanel pnlTop     = new JPanel();
    JPanel pnlBTN     = new JPanel();
	JPanel pnlCenter  = new JPanel();
	JPanel pnlBase    = new JPanel();
	JPanel pnlNormal  = new JPanel();
    JPanel pnlRB      = new JPanel();

    JLabel lblPercent = new JLabel();

    JButton btnI      = new JButton();
    JButton btnII     = new JButton();
    ButtonGroup group = new ButtonGroup();

    // Create an Observable

    notifier = new gridControlFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Frame

    this.setTitle("Brine Data");
    this.getContentPane().setLayout(new BorderLayout());

    // Top Panel

    pnlTop.setLayout(new BorderLayout());

    btnClose = new JButton(createImageIcon(cmn.cmnStruct.CLOSE));
    btnClose.setPreferredSize(new Dimension(29, 29));
    btnClose.setToolTipText("Close Dialog");
    btnClose.addActionListener(this);

    // -- 1st Separator Button

    btnI.setPreferredSize(new Dimension(4, 35));
    btnI.setText("");

    btnPlot = new JButton(createImageIcon(cmn.cmnStruct.PLOT));
    btnPlot.setPreferredSize(new Dimension(29, 29));
    btnPlot.setEnabled(false);
    btnPlot.setToolTipText("Begin Animation");
    btnPlot.addActionListener(this);

    // Center Panel

    if (iWidth < 750)
      iWidth = 750;

    // Base Panel

    pnlBase.setLayout(new BorderLayout());

    pData = new gridControlDataPanel( notifier, stStruct, stData, iWidth-20 );

    // Attach Widgets

    this.getContentPane().add(pnlTop,    BorderLayout.NORTH);
    pnlTop.add(pnlBTN,                   BorderLayout.WEST);
    pnlBTN.add(btnClose,                 null);
    pnlBTN.add(btnI,                     null);
    pnlBTN.add(btnPlot,                  null);

    this.getContentPane().add(pnlBase,   BorderLayout.SOUTH);
    pnlBase.add(pData,                   BorderLayout.NORTH);
//    pnlBase.add(pData,                   BorderLayout.CENTER);

    // Display the Frame

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();

    if (sID.equals("LAB"))
    {
//      this.setSize(new Dimension(iWidth, d.height-60));
      this.setSize(new Dimension(iWidth, 750));
	}
	else
	{
      this.setSize(new Dimension(iWidth, 500));
	}

    this.setLocation( (d.width  - this.getSize().width) / 2, 10 );

    this.setResizable(true);
    this.setVisible(true);
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
      System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /* ===================================================================== *
   * -------------------------- CLOSE DIALOG ----------------------------- *
   * ===================================================================== */

  /** METHOD gridControlFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class gridControlFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
      close();
    }
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
	sID         = null;
	stStruct    = null;
    stData      = null;
    notifier    = null;

    if (pData != null)
      pData.close();
    pData    = null;

    if (plot != null)
      plot.close();
    plot = null;

    btnClose     = null;
    btnPlot      = null;

    dispose();
  }

  /* ======================================================================= *
   * ----------------------------- GET METHODS ----------------------------- *
   * ======================================================================= */

  /** Method getWidth()
   *  <p> This method will determine the width of the Frame from the width of
   *      the table.
   *

  public int getWidth()
  {
	int iWidth = 0;

	if (iLength != null)
	{
	  if (iLength.length > 0)
	  {
		for (int i=0; i<iLength.length; i++)
		{
		  iWidth = iWidth + 8 * iLength[i];
		}
	  }
	}

	return (iWidth + 25);
  }
*/
  /* ======================================================================= *
   * ----------------------------- SET METHODS ----------------------------- *
   * ======================================================================= */

  /** Method setButtons()
   *  <p> This method will enable or disable the buttons depending on the data
   */

  private void setButtons()
  {
  }

  /* ======================================================================= *
   * --------------------------- DISPLAY FRAMES ---------------------------- *
   * ======================================================================= */

  /** Method beginAnimation()
   *  <p> This method will retrieve the data and begin the animation
   */

  private void beginAnimation()
  {
	gridPlotMappingListStruct st = null;

    st = pData.getData();

    if (plot != null)
      plot.close();
    plot = null;

    plot = new gridAnimationFrame( sID, st );
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnPlot)   { beginAnimation(); }
    if (event.getSource() == btnClose)  { close(); }
  }

  /* ===================================================================== *
   * ------------------------ OBSERVER ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg = new String( (String) arg);

    if (sArg.equals("RGB Curves Selected"))
    {
	  btnPlot.setEnabled(true);
	}

	if (sArg.equals("RGB Curves NOT Selected"))
	{
	  btnPlot.setEnabled(false);
	}
  }
}

/** CLASS gridControlFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class gridControlFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 02/25/2016
 *  @author  John Victorine
 */
