/*
 * @gridIO_Files.java Version 1.0 09/19/2013
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package grid.io;

import java.io.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

import util.utilFileIO;
import util.utilImageIO;

import grid.math.gridParametersStruct;   // Grid Parameters Structure
import grid.plot.gridPlotMappingStruct;  // Plot Mapping Structure
import iqstrat.iqstratHeadersListStruct; // Headers List Structure

/** Class gridIO_Files
 *  <p> This class will manage the plot output for the profile plot
 *
 *  @version 1.1 09/19/2013
 *  @author  John R. Victorine
 */

public class gridIO_Files
{
  /** Method getHomeDir()
   * <p> This method will retrieve the User's Home Directory
   * @return sDirectory = The User Home Directory
   */

  public static String getHomeDir()
  {
    String sDir = "";    // User Home Directory
    File   pDir = null;  // User Home Directory File Object

    try
    {
      pDir = new File( System.getProperty( "user.home" ) );
      sDir = new String( pDir.getCanonicalPath() );
    }
    catch (Exception e)
    {
      e.printStackTrace();
      String sError = new String("gridIO_Files.getHomeDir()\n "+e.toString());
      JOptionPane.showMessageDialog(
        (Component) null, sError, "ERROR", JOptionPane.ERROR_MESSAGE);
    }

    return (sDir);
  }

  /** Method createImage()
   * <p> this method will create an image from the Plot image
   *  @param sPath  = Directory Path for the Image file
   *  @param sName  = Filename of the Image file
   *  @param image  = Profile Plot Buffered Image
   */

  public static void createImage( String sPath, String sName, BufferedImage image )
  {
    String sOut  = "";

    sOut = sPath  + "/" + sName + ".png";

    utilImageIO pImage = new utilImageIO( image );
    pImage.save( sOut );

    image = null;
    sOut  = null;
  }

  /** Method createHTML()
   * <p> this method will create an html file from the Plot image
   *  @param sPath  = Directory Path for the Image file
   *  @param sName  = Filename of the Image file
   *  @param stWell = Well Header data Struct
   *  @param stMap  = Plot Mapping Structure
   */

  public static void createHTML( String sPath, String sName,
                                 iqstratHeadersListStruct stWell,
                                 gridParametersStruct     stGrid,
                                 gridPlotMappingStruct    stMap)
  {
    int           iError = util.utilFileIO.NOERROR;
    String        sOut   = sName + ".html";
    String        sHTML  = "";
    utilFileIO    pIO    = null;

    // Create HTML Document

    sHTML = new String(sHTML + "<html>\n");
    sHTML = new String(sHTML + "  <head>\n");
    sHTML = new String(sHTML + "    <title>Gridding Plot</title>\n");
    sHTML = new String(sHTML + "  </head>\n");

    sHTML = new String(sHTML + "  <body>\n");
    sHTML = new String(sHTML + "    <p>\n");
    sHTML = new String(sHTML + "    <img src=\"" + sName + ".png\">\n");

    sHTML = new String(sHTML + "    <p>\n");
    sHTML = AddWellList( sHTML, stWell, stMap );

    sHTML = new String(sHTML + "    <p>\n");
    sHTML = AddStatisticsData( sHTML, stMap );

    sHTML = new String(sHTML + "    <p>\n");
    sHTML = AddGriddingParameterData( sHTML, stGrid );

    sHTML = new String(sHTML + "    <p>\n");
    sHTML = AddPlotLimitsData( sHTML, stMap );

    sHTML = new String(sHTML + "  </body>\n");
    sHTML = new String(sHTML + "</html>\n");

    // Write HTML Document to Temporary File

    pIO  = new utilFileIO();
    pIO.Open(util.utilFileIO.DIRECTORY, util.utilFileIO.WRITE, sPath, sOut);
    pIO.Write(sHTML);

    if (iError == util.utilFileIO.NOERROR)
      pIO.Close();

    pIO.delete();
    pIO = null;

    util.BrowserControl.displayURL(sPath + "\\" + sOut);

    sOut  = null;
    sPath = null;
    sHTML = null;
  }

  /** Method AddWellList()
   * <p> This method will write the Project Well List Block
   * @param  sHTML    = The HTML File String
   * @param  stWell   = Well Header data Struct
   * @param stMap     = Plot Mapping Structure
   * @return sHTML    = The HTML File String
   */

  private static String AddWellList( String                   sHTML,
                                     iqstratHeadersListStruct stWell,
                                     gridPlotMappingStruct    stMap )
  {
	int i,j;
	double dataR = 0.0;
	double dataG = 0.0;
	double dataB = 0.0;

    // Well Header Information Block with User supplied Map Data.

    sHTML = new String(sHTML +
      "    <CENTER><H3> Oil &amp; Gas Well Data </H3></CENTER>\n");

    sHTML = new String(sHTML        +
          "<CENTER><TABLE BORDER=\"0\"      \n" +
          "               CELLPADDING=\"2\" \n" +
          "               CELLSPACING=\"2\" \n" +
          "               WIDTH = \"100%\">  \n");

    sHTML = new String(sHTML + "  <TR> \n");
    sHTML = new String(sHTML + "  <TD></TD>");
    sHTML = new String(sHTML + "  <TD><B>API Number</B></TD>");
    sHTML = new String(sHTML + "  <TD><B>Well Name</B></TD>");
    sHTML = new String(sHTML + "  <TD><B>Latitude</B></TD>");

    sHTML = new String(sHTML + "  <TD><B>Longitude</B></TD>");
	if (stMap.iRed > -1)
	{
      sHTML = new String(sHTML + "  <TD><B>" +
                         brine.brineStandardTools.CURVES[stMap.iRed][1] + "</B></TD>");
	}
    else
      sHTML = new String(sHTML + "  <TD><B>Data</B></TD>");

	if (stMap.iGreen > -1)
	{
      sHTML = new String(sHTML + "  <TD><B>" +
                         brine.brineStandardTools.CURVES[stMap.iGreen][1] + "</B></TD>");
	}
    else
      sHTML = new String(sHTML + "  <TD><B>Data</B></TD>");

	if (stMap.iBlue > -1)
	{
      sHTML = new String(sHTML + "  <TD><B>" +
                         brine.brineStandardTools.CURVES[stMap.iBlue][1] + "</B></TD>");
	}
    else
      sHTML = new String(sHTML + "  <TD><B>Data</B></TD>");

    sHTML = new String(sHTML + "  </TR> \n");

    if ((stWell != null) && (stMap != null))
    {
      for (i=0; i<stWell.iCount; i++)
      {
        sHTML = new String(sHTML + "  <TR> \n");
        sHTML = new String(sHTML + "  <TD>" + i + "</TD>");
        sHTML = new String(sHTML +
                "  <TD>" + stWell.stItem[i].sAPI  + "</TD>");
        sHTML = new String(sHTML +
                "  <TD>" + stWell.stItem[i].sName + "</TD>");
        sHTML = new String(sHTML +
                "  <TD>" + stWell.stItem[i].dLatitude  + "</TD>");
        sHTML = new String(sHTML +
                "  <TD>" + stWell.stItem[i].dLongitude  + "</TD>");

        for (j=0; j<stMap.iWells; j++)
        {
//		  if (stWell.stItem[i].sAPI.equals(stMap.sAPI[j]))
//		  if (stWell.stItem[i].sName.equals(stMap.sLease[j]))
		  if ((stWell.stItem[i].sName.equals(stMap.sLease[j]))||
		      ((stWell.stItem[i].sAPI.equals(stMap.sAPI[j])) && (stMap.sAPI[j].length() > 2)))
		  {
			if (stMap.iRed > -1)
			{
              sHTML = new String(sHTML + "  <TD>" + stMap.dZR[j] + "</TD>");
			}
			else
              sHTML = new String(sHTML + "  <TD>&nbsp;</TD>");

			if (stMap.iGreen > -1)
			{
              sHTML = new String(sHTML + "  <TD>" + stMap.dZG[j] + "</TD>");
			}
			else
              sHTML = new String(sHTML + "  <TD>&nbsp;</TD>");

			if (stMap.iBlue > -1)
			{
              sHTML = new String(sHTML + "  <TD>" + stMap.dZB[j] + "</TD>");
			}
			else
              sHTML = new String(sHTML + "  <TD>&nbsp;</TD>");
		  }
		}
        sHTML = new String(sHTML + "  </TR> \n");
	  }
	}

    sHTML = new String(sHTML + "</TABLE></CENTER> \n");
    sHTML = new String(sHTML        + "<P>");

	return (sHTML);
  }

  /** Method AddStatisticsData()
   *  <p> This method will add the Gridding Data to the Report.
   *  @param  sHTML  = HTML Text
   *  @param stMap   = Plot Mapping Structure
   *  @return sHTML  = HTML Text
   */

  private static String AddStatisticsData(String sHTML, gridPlotMappingStruct stMap)
  {
	int    i,j;
	int    iCount    = 0;
	int    iCurves[] = null;
	String stats[][] = null;
	double data      = 0.0;

    if (stMap.iRed   > -1) { iCount++; }
    if (stMap.iGreen > -1) { iCount++; }
    if (stMap.iBlue  > -1) { iCount++; }

    iCurves = new int[iCount];
    stats   = new String[iCount][10];
    iCount  = 0;

    if (stMap.iRed   > -1)
    {
	  iCurves[iCount] = stMap.iRed;
	  iCount++;
	}

    if (stMap.iGreen   > -1)
    {
	  iCurves[iCount] = stMap.iGreen;
	  iCount++;
	}

    if (stMap.iBlue   > -1)
    {
	  iCurves[iCount] = stMap.iBlue;
	  iCount++;
	}

    for (i=0; i<iCount; i++)
    {
	  stats[i][0] = new String( brine.brineStandardTools.CURVES[iCurves[i]][1] );
	  stats[i][1] = new String( brine.brineStandardTools.CURVES[iCurves[i]][2] );

	  for (j=2; j<10; j++)
	  {
		data = (double) ( (int) (100.0 * stMap.data[i][j-2] )/100.0 );
		stats[i][j] = new String( "" + data );
	  }
	}

    sHTML = new String(sHTML +
  "    <CENTER><H3> Data Statistics </H3></CENTER>\n");

    sHTML = new String(sHTML        +
          "<CENTER><TABLE BORDER=\"0\"      \n" +
          "               CELLPADDING=\"2\" \n" +
          "               CELLSPACING=\"2\" \n" +
          "               WIDTH = \"100%\">  \n");

    sHTML = new String(sHTML + "  <TR> \n");
    sHTML = new String(sHTML + "  <TD><B>MNEM</B></TD>");
    sHTML = new String(sHTML + "  <TD><B>Description</B></TD>");
    sHTML = new String(sHTML + "  <TD><B>Minimum</B></TD>");
    sHTML = new String(sHTML + "  <TD><B>5%</B></TD>");
    sHTML = new String(sHTML + "  <TD><B>25%</B></TD>");
    sHTML = new String(sHTML + "  <TD><B>Mean</B></TD>");
    sHTML = new String(sHTML + "  <TD><B>Median</B></TD>");
    sHTML = new String(sHTML + "  <TD><B>75%</B></TD>");
    sHTML = new String(sHTML + "  <TD><B>95%</B></TD>");
    sHTML = new String(sHTML + "  <TD><B>Maximum</B></TD>");
    sHTML = new String(sHTML + "  </TR> \n");

    for (i=0; i<iCount; i++)
    {
      sHTML = new String(sHTML + "  <TR> \n");
      sHTML = new String(sHTML + "  <TD>" + stats[i][0] + "</TD>");
      sHTML = new String(sHTML + "  <TD>" + stats[i][1] + "</TD>");
      sHTML = new String(sHTML + "  <TD>" + stats[i][2] + "</TD>");
      sHTML = new String(sHTML + "  <TD>" + stats[i][3] + "</TD>");
      sHTML = new String(sHTML + "  <TD>" + stats[i][4] + "</TD>");
      sHTML = new String(sHTML + "  <TD>" + stats[i][5] + "</TD>");
      sHTML = new String(sHTML + "  <TD>" + stats[i][6] + "</TD>");
      sHTML = new String(sHTML + "  <TD>" + stats[i][7] + "</TD>");
      sHTML = new String(sHTML + "  <TD>" + stats[i][8] + "</TD>");
      sHTML = new String(sHTML + "  <TD>" + stats[i][9] + "</TD>");
      sHTML = new String(sHTML + "  </TR> \n");
	}

    sHTML = new String(sHTML + "</TABLE></CENTER> \n");
    sHTML = new String(sHTML        + "<P>");

    return (sHTML);
  }

  /** Method AddGriddingParameterData()
   *  <p> This method will add the Gridding Data to the Report.
   *  @param  sHTML  = HTML Text
   *  @param  stGrid = Grid Parameters Data Structure
   *  @return sHTML  = HTML Text
   */

  private static String AddGriddingParameterData(String sHTML, gridParametersStruct stGrid)
  {
    sHTML = new String(sHTML +
  "    <CENTER><H3> Gridding Parameter &amp; Calculated Data </H3></CENTER>\n");

    sHTML = new String(sHTML        +
          "<CENTER><TABLE BORDER=\"0\"      \n" +
          "               CELLPADDING=\"2\" \n" +
          "               CELLSPACING=\"2\" \n" +
          "               WIDTH = \"650\">  \n");

    sHTML = new String(sHTML + "  <TR> \n");
    sHTML = new String(sHTML +
        "  <TD COLSPAN=2><B>Grid Area Parameters</B></TD> \n");
    sHTML = new String(sHTML + "  </TR> \n");

    sHTML = new String(sHTML + "  <TR> \n");
    sHTML = new String(sHTML +
        "  <TD>Minimum X in feet: " + stGrid.dXMin + "</TD> \n");
    sHTML = new String(sHTML +
        "  <TD>Minimum Y in feet: " + stGrid.dYMin + "</TD> \n");
    sHTML = new String(sHTML + "  </TR> \n");

    sHTML = new String(sHTML + "  <TR> \n");
    sHTML = new String(sHTML +
        "  <TD>Maximum X in feet: " + stGrid.dXMax + "</TD> \n");
    sHTML = new String(sHTML +
        "  <TD>Maximum Y in feet: " + stGrid.dYMax + "</TD> \n");
    sHTML = new String(sHTML + "  </TR> \n");

    sHTML = new String(sHTML + "  <TR> \n");
    sHTML = new String(sHTML +
        "  <TD>Number of Columns: " + stGrid.iXCells + "</TD> \n");
    sHTML = new String(sHTML +
        "  <TD>Number of Rows: "    + stGrid.iYCells + "</TD> \n");
    sHTML = new String(sHTML + "  </TR> \n");

    sHTML = new String(sHTML + "  <TR> \n");
    sHTML = new String(sHTML + "  <TD COLSPAN=2>Minimum Grid Spacing: " +
                                  stGrid.dCellSize + "</TD> \n");
    sHTML = new String(sHTML + "  </TR> \n");

    sHTML = new String(sHTML + "</TABLE></CENTER> \n");
    sHTML = new String(sHTML        + "<P>");

    sHTML = new String(sHTML        +
          "<CENTER><TABLE BORDER=\"0\"      \n" +
          "               CELLPADDING=\"0\" \n" +
          "               CELLSPACING=\"0\" \n" +
          "               WIDTH = \"650\">  \n");

    sHTML = new String(sHTML + "  <TR> \n");
    sHTML = new String(sHTML +
        "  <TD COLSPAN=\"4\"><B>Search Parameter Selection</B></TD> \n");
    sHTML = new String(sHTML + "  </TR> \n");

    sHTML = new String(sHTML + "  <TR> \n");
    sHTML = new String(sHTML +
        "  <TD>Inverse Distance Weighting Exponent: </TD> \n");
    sHTML = new String(sHTML + "  <TD>" + stGrid.dExponent + "</TD> \n");
    sHTML = new String(sHTML +
        "  <TD>Maximum Distance to Nearest data point, ft: </TD> \n");
    sHTML = new String(sHTML + "  <TD>" + stGrid.dMaxDistTo1st + "</TD> \n");
    sHTML = new String(sHTML + "  </TR> \n");

    sHTML = new String(sHTML + "  <TR> \n");
    sHTML = new String(sHTML + "  <TD>Number of Nearest Neighbors: </TD> \n");
    sHTML = new String(sHTML + "  <TD>" +stGrid.iNeighbors + "</TD> \n");
    sHTML = new String(sHTML + "  <TD>Maximum Search Radius, ft: </TD> \n");
    sHTML = new String(sHTML +
        "  <TD>" + stGrid.dMaxSearchRadius + "</TD> \n");
    sHTML = new String(sHTML + "  </TR> \n");

    sHTML = new String(sHTML + "</TABLE></CENTER> \n");
    sHTML = new String(sHTML        + "<P>");

    return (sHTML);
  }

  /** Method AddPlotLimitsData()
   *  <p> This method will add the Gridding Data to the Report.
   *  @param  sHTML  = HTML Text
   *  @param  stGrid = Grid Parameters Data Structure
   *  @return sHTML  = HTML Text
   */

  private static String AddPlotLimitsData(String sHTML, gridPlotMappingStruct stMap)
  {
    sHTML = new String(sHTML +
  "    <CENTER><H3> Colorlith Plot Limits </H3></CENTER>\n");

    sHTML = new String(sHTML        +
          "<CENTER><TABLE BORDER=\"1\"      \n" +
          "               CELLPADDING=\"2\" \n" +
          "               CELLSPACING=\"2\" \n" +
          "               WIDTH = \"100%\">  \n");

    sHTML = new String(sHTML + "  <TR> \n");
    sHTML = new String(sHTML + "  <TD Colspan=3><B>&nbsp;</B></TD>");
    sHTML = new String(sHTML + "  <TH Colspan=2><B>Minimum</B></TH>");
    sHTML = new String(sHTML + "  <TH Colspan=2><B>Maximum</B></TH>");
    sHTML = new String(sHTML + "  </TR> \n");

    sHTML = new String(sHTML + "  <TR> \n");
    sHTML = new String(sHTML + "  <TD><B>MNEM</B></TD>");
    sHTML = new String(sHTML + "  <TD><B>Description</B></TD>");
    sHTML = new String(sHTML + "  <TD><B>Color</B></TD>");
    sHTML = new String(sHTML + "  <TD><B>BrineData </B></TD>");
    sHTML = new String(sHTML + "  <TD><B>Color Value</B></TD>");
    sHTML = new String(sHTML + "  <TD><B>Brine Data </B></TD>");
    sHTML = new String(sHTML + "  <TD><B>Color Value</B></TD>");
    sHTML = new String(sHTML + "  </TR> \n");

    if (stMap.iRed   > -1)
    {
      sHTML = new String(sHTML + "  <TR> \n");
      sHTML = new String(sHTML + "  <TD>" +
	            brine.brineStandardTools.CURVES[stMap.iRed][1] + "</TD>");
	  sHTML = new String(sHTML + "  <TD>" +
	            brine.brineStandardTools.CURVES[stMap.iRed][2] + "</TD>");
	  sHTML = new String(sHTML + "  <TD>RED</TD>");

	  sHTML = new String(sHTML + "  <TD>" + stMap.dMinR + "</TD>");
	  sHTML = new String(sHTML + "  <TD>255</TD>");
	  sHTML = new String(sHTML + "  <TD>" + stMap.dMaxR + "</TD>");
	  sHTML = new String(sHTML + "  <TD>0</TD>");
      sHTML = new String(sHTML + "  </TR> \n");
	}

    if (stMap.iGreen   > -1)
    {
      sHTML = new String(sHTML + "  <TR> \n");
      sHTML = new String(sHTML + "  <TD>" +
	            brine.brineStandardTools.CURVES[stMap.iGreen][1] + "</TD>");
	  sHTML = new String(sHTML + "  <TD>" +
	            brine.brineStandardTools.CURVES[stMap.iGreen][2] + "</TD>");
	  sHTML = new String(sHTML + "  <TD>GREEN</TD>");

	  sHTML = new String(sHTML + "  <TD>" + stMap.dMinG + "</TD>");
	  sHTML = new String(sHTML + "  <TD>255</TD>");
	  sHTML = new String(sHTML + "  <TD>" + stMap.dMaxG + "</TD>");
	  sHTML = new String(sHTML + "  <TD>0</TD>");
      sHTML = new String(sHTML + "  </TR> \n");
	}

    if (stMap.iBlue   > -1)
    {
      sHTML = new String(sHTML + "  <TR> \n");
      sHTML = new String(sHTML + "  <TD>" +
	            brine.brineStandardTools.CURVES[stMap.iBlue][1] + "</TD>");
	  sHTML = new String(sHTML + "  <TD>" +
	            brine.brineStandardTools.CURVES[stMap.iBlue][2] + "</TD>");
	  sHTML = new String(sHTML + "  <TD>BLUE</TD>");

	  sHTML = new String(sHTML + "  <TD>" + stMap.dMinB + "</TD>");
	  sHTML = new String(sHTML + "  <TD>255</TD>");
	  sHTML = new String(sHTML + "  <TD>" + stMap.dMaxB + "</TD>");
	  sHTML = new String(sHTML + "  <TD>0</TD>");
      sHTML = new String(sHTML + "  </TR> \n");
	}

    sHTML = new String(sHTML + "</TABLE></CENTER> \n");
    sHTML = new String(sHTML        + "<P>");

    return (sHTML);
  }
}

/*
 *  @version 1.1 09/19/2013
 *  @author  John Victorine
 */
