/*
 * @(#)grid.java Version 1.00, 28 June 2001
 *
 * Copyright (c) 2001 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, USA
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of the
 * Kansas Geological Survey.  ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with the Kansas Geological Survey.
 *
 *       Last change:  GCB   2 Jul 2001    4:12 pm
 *                     JRV  25 Nov 2003    7:10 am
 *                     - Changed the location to grid.math from pfeffer.math
 *                       and name of the file from pfefferGrid.java
 *                       to grid.java. Modified the comments to match
 *                       GEMINI Standards.
 */

package grid.math;

import java.awt.*;

import grid.math.gridParameters;
import grid.math.gridSearchParameters;

/**
 * Defines a 2D grid structure and inverse-distance interpolation routine.
 *
 * @version 1.00, 28 June 2001
 * @author Geoff Bohling
 */

public class grid
{
  /** The grid-specification parameters */

  public gridParameters GridPar;

  /** The nearest-neighbor search parameters & distance exponent */

  public gridSearchParameters SearchPar;

  /** The grid of z values; zValues[i][j] is value at xmin+i*dx, ymin+j*dy */

  public double[][] zValues;

  /** Constructor grid()
   * <p>Constructor taking grid parameter and search parameter specifications
   * @param pGP = Defines grid specification parameters for a rectangular
   *              X,Y grid. Grid parameters are private: "set" routines used
   *              to maintain consistent grid specification.
   * @param pSP = Defines nearest-neighbor search parameters and exponent for
   *              inverse-distance interpolation.
   */

  public grid(gridParameters pGP, gridSearchParameters pSP)
  {
    GridPar = pGP;
    SearchPar = pSP;
    //zValues = new double[GridPar.nx()][GridPar.ny()];
  }

  /** Constructor grid()
   * <p>Constructor taking x,y,z vectors; sets up default parameters
   *    and interpolates
   * @param x = Array of x location vectors
   * @param y = Array of y location vectors
   * @param z = Array of z data vectors
   */

  public grid( double[] x, double[] y, double[] z )
  {
    GridPar   = new gridParameters( x, y );
    SearchPar = new gridSearchParameters( x, y );
    //zValues = new double[GridPar.nx()][GridPar.ny()];
    InvDistInterpolate( x, y, z );
  }

  /** Constructor grid()
   * <p>Constructor taking x,y,z vectors and specified grid, search parameters;
   *    does interpolation
   * @param x   = Array of x location vectors
   * @param y   = Array of y location vectors
   * @param z   = Array of z data vectors
   * @param pGP = Defines grid specification parameters for a rectangular
   *              X,Y grid. Grid parameters are private: "set" routines used
   *              to maintain consistent grid specification.
   * @param pSP = Defines nearest-neighbor search parameters and exponent for
   *              inverse-distance interpolation.
   */

  public grid( double[] x,
               double[] y,
               double[] z,
               gridParameters pGP,
               gridSearchParameters pSP )
  {
    GridPar = pGP;
    SearchPar = pSP;
    //zValues = new double[GridPar.nx()][GridPar.ny()];
    InvDistInterpolate( x, y, z );
  }

  /** Method InvDistInterpolate()
   * <p>Inverse-distance^expo interpolation routine; interpolates z values from
   * x,y data locations to regular grid (specified in GridPar) using
   * nearest-neighbor search parameters and distance exponent given in
   * SearchPar.  "Search failure" occurs at a grid node if nearest data point
   * is outside maximum allowed distance to 1st nearest neighbor
   * (SearchPar.rad1stMax()) or if no data points are found within maximum
   * search radius (SearchPar.radSrchMax).  Search failure is indicated by a
   * z value of NaN in the zValues matrix.
   * @param x   = Array of x location vectors
   * @param y   = Array of y location vectors
   * @param z   = Array of z data vectors
   */

  public void InvDistInterpolate( double[] x, double[] y, double[] z )
  {
    int nData = x.length;

    // Transfer grid and search parameters to local variables

    double xmin = GridPar.xmin();
    double xmax = GridPar.xmax();
    double dx   = GridPar.dx();
    int    nx   = GridPar.nx();
    double ymin = GridPar.ymin();
    double ymax = GridPar.ymax();
    double dy   = GridPar.dy();
    int    ny   = GridPar.ny();
    int    nnn  = SearchPar.nnn();
    double rad1 = SearchPar.rad1stMax();
    double rad2 = SearchPar.radSrchMax();
    double expo = SearchPar.dexpo();

    // Initialize matrix of z values

    zValues = new double[nx][ny];

    // squared-distance threshold for deciding whether grid node coincides
    // with data point

    double lild2 = Math.pow( 0.01*dx, 2 );

    // grid node coordinates

    double gx, gy;

    // vector squared distances from current grid node to data points

    double[] dst2 = new double[nData];

    // sorting index vector for dst2

    int[] iord = new int[nData];
    for ( int i=0; i<nData; i++)
    {
      iord[i] = i;
    }

    // numerator, denominator in grid value computation, distance^expo

    double snum, sden, dpow;

    int jinc = -1;                 // increment direction for j (y-coord. index)
    for ( int i=0; i<nx; i++)
    {
      gx = xmin + i*dx;		   // current grid node x coordinate switch

      /* switch increment direction for j so that "lace" through the grid
       * (first up one column, then down the next), so that distances to
       * data points never change abruptly
       */

      jinc = (-jinc);

      for ( int j=( jinc>0 ? 0 : (ny-1) ); ( jinc>0 ? j<ny : j>=0 ); j+=jinc)
      {
        gy = ymin + j*dy;         // current grid node y coordinate

        // compute squared distances from grid node to data points

        for ( int k=0; k<nData; k++)
	    {
          dst2[k] = Math.pow( x[k]-gx, 2 ) + Math.pow( y[k]-gy, 2 );
        }

        /* recompute iord, sorting index vector for dst2; should not
         * require many iterations, since distances change only a little
         * as we move through the grid
         */

        int nswaps, itmp;
        do
        {
          nswaps = 0;
          for ( int k=0; k<(nData-1); k++ )
          {
            if ( dst2[iord[k]] > dst2[iord[k+1]] )
            {
              itmp = iord[k+1];
              iord[k+1] = iord[k];
              iord[k] = itmp;
              nswaps++;
            }
          }
        } while ( nswaps>0 );

        // compute grid node z value:

        if ( dst2[iord[0]] > Math.pow( rad1, 2) )
        {
          // nearest data point too far away; set grid value to NaN

          zValues[i][j] = Double.NaN;
        }
        else if ( dst2[iord[0]] < lild2 )
        {
          // nearest data point very close; just use that z value

          zValues[i][j] = z[iord[0]];
        }
        else
        {
          // accumulate snum, sden over nnn nearest neighbors

          snum = 0.0;
          sden = 0.0;
          for ( int k=0; k<nnn; k++)
          {
            // if outside max search radius, quit accumulating

            if ( dst2[iord[k]] > Math.pow( rad2, 2 ) ) break;

            // dpow is grid node-data point distance raised to expo

            dpow = Math.pow( dst2[iord[k]], 0.5*expo );

            snum += z[iord[k]]/dpow;    // add scaled z value into numerator
            sden += 1.0/dpow;           // add weight into denominator
          }

          /* compute grid node z value; sden will be zero if no neighbors
           * were found within rad2; if sden=0, zValue[i][j] set to NaN,
           * indicating search failure
           */

          if ( sden>0.0 )
          {
            zValues[i][j] = snum/sden;
          }
          else
          {
            zValues[i][j] = Double.NaN;
          }
        }
      }
    }
  }
}